//
//
// BeInternetCentral Lib
//
// ©1997 Hubert Figuière
//
// May 27th 1997
//
// BICInternals.cpp


#include "lib/BICInternals.h"


//storage field names
const char * PERSONALITY_ATTRIBUTE_NAME = "PersonalityAttr";
const char * DEFAULT_PERSONALITY_ATTRIBUTE_NAME = "DefaultPersonalityAttr";
const char * HELPERS_ATTRIBUTE_NAME = "HelpersAttr";

const char * DEFAULT_PERSONALITY_VALUE = "Default";


////////////////////////////////////////////////////////////
BICInternals::BICInternals()
{
	mInitResult = B_NO_INIT;
	mRegDatas = NULL;
	mDefaultPersonality = (key_attribute_id)malloc((KEY_NAME_LENGTH + 1) * sizeof(char));
	if (mDefaultPersonality == NULL) {
		mInitResult = BIC_NO_MEMORY;
	}
	else if (Lock()) {
		mRegDatas = new BMessage(B_SIMPLE_DATA);
		// look up for defa	ult registry file
		mInitResult = ResetRegPath();		
		//installer la notification
		Unlock();
	}
}


////////////////////////////////////////////////////////////
BICInternals::~BICInternals()
{
	Lock();
	if (mDefaultPersonality != NULL) {
		free(mDefaultPersonality);
	}
	delete mRegDatas;
	Unlock();
}
#pragma mark -


////////////////////////////////////////////////////////////
bool BICInternals::WriteDatas()
{
	status_t err;
	
	Lock();
	err =  SaveDatas(mRegistryFile, mRegDatas);
	Unlock();
	
	return err;
}



////////////////////////////////////////////////////////////
status_t BICInternals::CountPersonality(int32 & count)
{
	//doc
	status_t err;
	type_code itemType;

	Lock();
	if (!mIsLoaded) {
		Unlock();
		return BIC_NO_INIT;
	}
	
	err = mRegDatas->GetInfo(PERSONALITY_ATTRIBUTE_NAME, &itemType, &count);
	if ((err != B_NO_ERROR) && (itemType != B_STRING_TYPE)) {
		//ensure that it is a string...
		count = 0;
		err = BIC_ATTRIBUTE_NOT_FOUND;
	}

	Unlock();	
	return err;
}


////////////////////////////////////////////////////////////
status_t BICInternals::SetDefaultPersonality(const key_attribute_id personality)
{
	status_t err;
	int32 count, i;
	bool hasPersonality;
	char * str;

	Lock();
	strncpy(mDefaultPersonality, personality, KEY_NAME_LENGTH);
	if (mRegDatas->HasString(DEFAULT_PERSONALITY_ATTRIBUTE_NAME)) {
		err = mRegDatas->ReplaceString(DEFAULT_PERSONALITY_ATTRIBUTE_NAME, mDefaultPersonality);
	}
	else {
		err = mRegDatas->AddString(DEFAULT_PERSONALITY_ATTRIBUTE_NAME, mDefaultPersonality);
	}
	/*
		check if this personality exists... if not, should be added... since default personality
		should be available
	*/	
	hasPersonality = false;		
	err = CountPersonality(count);
	if (err == B_NO_ERROR) {
		/*find the personality...*/
		for (i = 0; i < count; i++) {
			err = mRegDatas->FindString(PERSONALITY_ATTRIBUTE_NAME, i, &str);
			if (strcmp(str, mDefaultPersonality) == 0) {
				hasPersonality = true;
				break;
			}
		}
	}
	if (!hasPersonality) {
		AddPersonality(mDefaultPersonality);
	}
	
	Unlock();	
	return err;
}


////////////////////////////////////////////////////////////
status_t BICInternals::GetPersonality(const int32 index, key_attribute_id personality, ssize_t & size)
{
	//doc
	status_t err;
	char * str;
	ssize_t strSize;

	Lock();	
	err = mRegDatas->FindString(PERSONALITY_ATTRIBUTE_NAME, index, &str);
	if (err == B_NO_ERROR) {
		strSize = strlen(str);
		if (strSize <= size) {
			strcpy(personality, str);
			size = strSize;
		}
		else {
			personality[0] = 0;
			size = strSize;
			err = BIC_BUFFER_TOO_SMALL;
		}
	}
	else {
		err = BIC_ATTRIBUTE_NOT_FOUND;
	}

	Unlock();	
	return err;
}


////////////////////////////////////////////////////////////
status_t BICInternals::AddPersonality(const key_attribute_id personality)
{
	status_t err;
	
	Lock();
	err = mRegDatas->AddString(PERSONALITY_ATTRIBUTE_NAME, personality);
	Unlock();
	
	return err;
}


////////////////////////////////////////////////////////////
status_t BICInternals::DeletePersonality(const key_attribute_id personality)
{
	status_t err;
	int32 count, i;
	key_attribute_id str;

	err = CountPersonality(count);

	Lock();	
	if (err == B_NO_ERROR) {
		/*find the personality...*/
		for (i = 0; i < count; i++) {
			err = mRegDatas->FindString(PERSONALITY_ATTRIBUTE_NAME, i, &str);
			if (strcmp(str, mDefaultPersonality) == 0) {
				err = mRegDatas->RemoveData(PERSONALITY_ATTRIBUTE_NAME, i);
				/* retirer toutes les données qui lui sont assignées...*/
				err = RemoveAssociatedKeys(personality);
				break;
			}
		}
	}
	Unlock();	

	return err;
}

#pragma mark -

////////////////////////////////////////////////////////////
status_t BICInternals::GetKey(const key_id id, const key_attribute_id attribute, key_value value, size_t & size)
{	
	//doc
	ssize_t bufSize = size;
	status_t err;
	key_id realKey;
	ssize_t neededSize = strlen(attribute) + strlen(id) + 1;
	key_value str;

	value[0] = 0;			
	size = 0;
	
	realKey = (key_id)malloc((neededSize + 1) * sizeof(char));
	if (realKey == NULL) {
		return BIC_NO_MEMORY;
	}

	sprintf(realKey, "%s/%s", id, attribute);

	Lock();
	err = mRegDatas->FindString(realKey, &str);
	if (err == B_NO_ERROR) {
		neededSize = strlen(str);
		if (neededSize <= bufSize) {
			/*check to see whether strcpy returns the num of char copied or not*/
			size = strlen(str);
			strcpy(value, str);
		}
		else {
			err = BIC_BUFFER_TOO_SMALL;
		}	
		size = neededSize;
	}
	else {
		err = BIC_KEY_NOT_FOUND;
	}
	
	Unlock();
	
	free(realKey);
	
	return err;
}



////////////////////////////////////////////////////////////
status_t BICInternals::SetKey(const key_id id, const key_attribute_id attribute, const key_value value)
{
	bool replace;
	status_t err;
	key_id realKey;
	ssize_t neededSize = strlen(attribute) + strlen(id) + 1;
	
	realKey = (key_id)malloc((neededSize + 1) * sizeof(char));
	if (realKey == NULL) {
		return B_NO_MEMORY;
	}

	sprintf(realKey, "%s/%s", id, attribute);
	
	Lock();

	replace = mRegDatas->HasString(realKey);
	if (replace) {
		err = mRegDatas->ReplaceString(realKey, value);
	}
	else {
		err = mRegDatas->AddString(realKey, value);
	}
	
	Unlock();
	
	free(realKey);
	
	return err;
}


////////////////////////////////////////////////////////////
status_t BICInternals::DeleteKey(const key_id id, const key_attribute_id attribute)
{
	//doc
	status_t err;
	key_id realKey;
	ssize_t realKeySize = strlen(id) + strlen(attribute) + 1;
	
	realKey = (key_id)malloc(realKeySize + 1);
	if (realKey == NULL) {
		return B_NO_MEMORY;
	}
	
	sprintf(realKey, "%s/%s", id, attribute);

	Lock();
	err = mRegDatas->RemoveName(realKey);
	Unlock();

	free(realKey);
	
	return err;
}

#pragma mark -


////////////////////////////////////////////////////////////
status_t BICInternals::SetRegPath(const char * dir, const char * name)
{
	//doc
	status_t err;
	char * str;
	
	BPath regPath(dir, name, true);
	
	if ((name == NULL) || (name[0] == 0)) {
		regPath.Append(_BIC_DEFAULT_REGISTRY_NAME_);
	}
	
	err = regPath.InitCheck();
	if (err != B_NO_ERROR) {
		return err;
	}
	err = SetUpDatasFile(regPath);
	if (err != B_NO_ERROR) {
		return err;
	}

	Lock();
	err = LoadDatas(mRegistryFile, mRegDatas, true);
	if (err != B_NO_ERROR) {
		Unlock();
		return err;
	}

	// set up default datas...
	if (mRegDatas->FindString(DEFAULT_PERSONALITY_ATTRIBUTE_NAME, &str) == B_NO_ERROR) {
		strcpy(mDefaultPersonality, str);
	}
	else {
		/*err = */
		SetDefaultPersonality(DEFAULT_PERSONALITY_VALUE);
	}
	Unlock();

	return err;
}



////////////////////////////////////////////////////////////
status_t BICInternals::GetRegPath(const char * dir, char * name)
{
	status_t err = B_NO_ERROR;
	
	//************
		
	return err; 
}


////////////////////////////////////////////////////////////
status_t BICInternals::ResetRegPath(void)
{
	//doc
	status_t err;
	BPath userConfigPath;
	
	err = find_directory(B_USER_CONFIG_DIRECTORY, &userConfigPath, true);
	if (err == B_NO_ERROR) {
		err = SetRegPath(userConfigPath.Path(), _BIC_DEFAULT_REGISTRY_NAME_);
	}
	
	return err;
}


////////////////////////////////////////////////////////////
status_t BICInternals::SaveRegistry(void)
{
	status_t err;
	
	Lock();
	err = SaveDatas(mRegistryFile, mRegDatas);
	Unlock();
	
	return err;
}

#pragma mark -


////////////////////////////////////////////////////////////
/*multi-thread access...*/
bool BICInternals::Lock()
{
	//doc
	return mLock.Lock();
}



////////////////////////////////////////////////////////////
void BICInternals::Unlock()
{
	//doc
	mLock.Unlock();
}



////////////////////////////////////////////////////////////
bool BICInternals::IsLocked()
{
	//doc
	return mLock.IsLocked();
}


#pragma mark -


////////////////////////////////////////////////////////////
status_t BICInternals::SetUpDatasFile(const BPath & path)
{
	status_t err;
	
	Lock();
	mRegistryFile.SetTo(path.Path(), true);

	err = mRegistryFile.InitCheck();
	Unlock();
		
	return err;
}



////////////////////////////////////////////////////////////
status_t BICInternals::SaveDatas(const BEntry & file, const BMessage * datas)
{
	status_t	err = B_NO_ERROR;
	bool		result = false;
	char *		buf = NULL;
	ssize_t		dataSize = 0;
	BFile		myFile(&file, B_READ_WRITE | B_CREATE_FILE);
	
	dataSize = datas->FlattenedSize();
	
	buf = (char *)malloc(dataSize);
	err = datas->Flatten(&myFile);
	printf("BICInternals::SaveDatas - datas->Flatten = %ld\n", err);
	
	if (buf != NULL) {
		free(buf);
	}

	return err;
}



////////////////////////////////////////////////////////////
status_t BICInternals::LoadDatas(const BEntry & file, BMessage * datas, bool create)
{
	status_t	err = B_NO_ERROR;
	char *		buf = NULL;
	ssize_t		dataSize = 0;
	BFile		myFile(&file, B_READ_ONLY);			//we create the file because we need it...
	
	err = myFile.InitCheck();
	if (err != B_NO_ERROR) {
		printf("BICInternals::LoadDatas - myFile.InitCheck = %lx\n", err);
		//create the registry if it doesn't exists...
		if ((create) && (err == ENOENT)) {
			err = myFile.SetTo(&file, B_READ_WRITE | B_CREATE_FILE);
			if (err != B_NO_ERROR) {
				printf("BICInternals::LoadDatas - myFile.SetTo = %lx\n", err);	
				return err;
			}
			BMessage * msg = new BMessage(B_SIMPLE_DATA);
			//**** handle errors here...
			err = msg->Flatten(&myFile);
			err = myFile.Seek(0, SEEK_SET);
			delete msg;
		}
		else {
			return err;
		}
	}
	
	buf = (char *)malloc(dataSize);
	if (buf == NULL) {
		return B_NO_MEMORY;
	}

	err = datas->Unflatten(&myFile);
	printf("BICInternals::LoadDatas - datas->Unflatten = %ld\n", err);	
	datas->PrintToStream();

	if (buf != NULL) {
		free(buf);
	}
	
	return err;
}


////////////////////////////////////////////////////////////
status_t BICInternals::RemoveAssociatedKeys(const key_attribute_id attribute)
{
	//************
	
	return B_NO_ERROR;
}



////////////////////////////////////////////////////////////
