//
//
// BeInternetCentral Lib
//
// ©1997 Hubert Figuière
//
// May 27th 1997
//
// BICInternals.h

#ifndef _BIC_INTERNALS_H_
#define _BIC_INTERNALS_H_

#include "lib/libbic.internals.h"


class BICInternals
{
	public:
		BICInternals();
		virtual ~BICInternals();
		
		status_t InitCheck() const { return mInitResult; };
		bool WriteDatas();
		
		status_t CountPersonality(int32 & count);
		char * GetDefaultPersonality() const { return mDefaultPersonality; };
		status_t SetDefaultPersonality(const key_attribute_id personality);
		status_t GetPersonality(const int32 index, key_attribute_id personality, ssize_t & size);
		status_t AddPersonality(const key_attribute_id personality);
		status_t DeletePersonality(const key_attribute_id personality);
	
		status_t GetKey(const key_id id, const key_attribute_id attribute, key_value value, size_t & size);
		status_t SetKey(const key_id id, const key_attribute_id attribute, const key_value value);
		status_t DeleteKey(const key_id id, const key_attribute_id attribute);
		
		/*registry access... might be changed in future implementations */
		status_t SetRegPath(const char * dir, const char * name);
		status_t GetRegPath(const char * dir, char * name);
		status_t ResetRegPath(void);
		status_t SaveRegistry(void);

		/*multi-thread access...*/
		bool Lock();
		void Unlock();
		bool IsLocked();
	protected:
		status_t SetUpDatasFile(const BPath & path);
		status_t SaveDatas(const BEntry & file, const BMessage * datas);
		status_t LoadDatas(const BEntry & file, BMessage * datas, bool create = false);
		/*attribute/key management*/
		status_t RemoveAssociatedKeys(const key_attribute_id attribute);
		status_t RenameAssociatedKeys(const key_attribute_id oldAttribute, const key_attribute_id newAttribute);
	private:
		bool	mIsLoaded;
		status_t	mInitResult;
		BLocker mLock;
		BEntry	mRegistryFile;
		BMessage * mRegDatas;
		char * mDefaultPersonality;
};

#endif
